#!/usr/bin/python3
"""
## © University of Southampton IT Innovation Centre, 2018
##
## Copyright in this software belongs to University of Southampton
## IT Innovation Centre of Gamma House, Enterprise Road, 
## Chilworth Science Park, Southampton, SO16 7NS, UK.
##
## This software may not be used, sold, licensed, transferred, copied
## or reproduced in whole or in part in any manner or form or in or
## on any media by any person other than in accordance with the terms
## of the Licence Agreement supplied with the software, or otherwise
## without the prior written consent of the copyright owners.
##
## This software is distributed WITHOUT ANY WARRANTY, without even the
## implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE, except where stated in the Licence Agreement supplied with
## the software.
##
##      Created By :            Michael Boniface
##      Created Date :          20-03-2018
##      Updated By :            Nikolay Stanchev
##      Updated Date :          30-08-2018
##      Created for Project :   FLAME
"""

import subprocess


def test_write_telegraf_conf():
    # test telegraf monitoring configuration
    TELEGRAF_CONF_DIR = "/etc/telegraf"
    LOCATION = "DC1"
    SFC_ID = "media_service_A"
    SFC_INSTANCE_ID = "media_service_A_instance"
    SF_PACKAGE_ID = "streaming_service"
    SF_ID = "streaming_service_instance"
    SF_ENDPOINT_ID = "endpoint"
    INFLUXDB_URL = "http://172.29.236.10"
    DATABASE_NAME = SFC_ID

    # mk telegraf conf directory
    cmd = 'sudo mkdir -p /etc/telegraf'
    (out, err, code) = run_command(cmd)
    assert code == 0, "Failed to create telegraf conf dir : " + str(code) + ", cmd=" + cmd

    cmd = 'sudo mkdir -p /etc/telegraf/telegraf.d'
    (out, err, code) = run_command(cmd)
    assert code == 0, "Failed to create telegraf include dir : " + str(code) + ", cmd=" + cmd

    # run write config template script with no telegraf.d directory
    cmd = 'sudo cp /vagrant/scripts/clmc-agent/telegraf.conf /etc/telegraf/'
    (out, err, code) = run_command(cmd)
    assert code == 0, "Failed to copy telegraf.conf : " + str(code) + ", cmd=" + cmd

    cmd = 'sudo cp /vagrant/scripts/clmc-agent/telegraf_output.conf /etc/telegraf/telegraf.d/'
    (out, err, code) = run_command(cmd)
    assert code == 0, "Failed to copy telegraf_output.conf : " + str(code) + ", cmd=" + cmd

    # run template relacement script with incorrect arguments
    cmd = 'sudo /vagrant/scripts/clmc-agent/configure.sh'
    (out, err, code) = run_command(cmd)
    assert code == 1, "Failed to return error on incorrect arguments : " + str(code) + ", cmd=" + cmd

    # run template relacement script with all arguments
    cmd = 'sudo /vagrant/scripts/clmc-agent/configure.sh ' + LOCATION + ' ' + SFC_ID + ' ' + SFC_INSTANCE_ID + ' ' + SF_PACKAGE_ID + ' ' + SF_ID + ' ' + SF_ENDPOINT_ID + ' ' + INFLUXDB_URL
    (out, err, code) = run_command(cmd)
    assert code == 0, "Configure command returned error, output=" + str(out) + ", cmd=" + cmd

    # check that replacement was correct in telegraf.conf
    TELEGRAF_GENERAL_CONF_FILE = TELEGRAF_CONF_DIR + "/telegraf.conf"
    try:
        with open(TELEGRAF_GENERAL_CONF_FILE) as general_conf:
            lines = general_conf.read()
            assert lines.find(LOCATION), "Cannot find location"
            assert lines.find(SFC_ID), "Cannot find sfc_id"
            assert lines.find(SFC_INSTANCE_ID), "Cannot find sfc_instance_id"
            assert lines.find(SF_PACKAGE_ID), "Cannot find sf_package_id"
            assert lines.find(SF_ID), "Cannot find sf_id"
            assert lines.find(SF_ENDPOINT_ID), "Cannot find sf_endpoint_id"
    except FileNotFoundError:
        assert False, "Telegraf general conf file not found, " + TELEGRAF_GENERAL_CONF_FILE

    # check that replacement was correct in telegraf_output.conf
    TELEGRAF_OUTPUT_CONF_FILE = TELEGRAF_CONF_DIR + "/telegraf.d/telegraf_output.conf"
    try:
        with open(TELEGRAF_OUTPUT_CONF_FILE) as output_conf:
            lines = output_conf.read()
            assert lines.find(INFLUXDB_URL), "Cannot find influx_db url"
            assert lines.find(DATABASE_NAME), "Cannot find database"
    except FileNotFoundError:
        assert False, "Telegraf output conf file not found, " + TELEGRAF_OUTPUT_CONF_FILE


def run_command(cmd):
    """
    Run a shell command. Wrapper for executing commands on the cli, returning (std_out, std_error, process_return_code)

    :param cmd: {string} -- command to run in the shell

    :return: stdout, stderr, exit code -- tuple of the process's stdout, stderr and exit code (0 on success)
    """

    proc = subprocess.Popen([cmd], stdout=subprocess.PIPE, shell=True)
    out, err = proc.communicate()
    return_code = proc.returncode
    return out, err, return_code
